# -*- coding: utf-8 -*-
from setuphelpers import *
import platform
import json

uninstallkey = []

# Declaring specific app values (TO CHANGE)
bin_name_deb_string = "rocketchat_%s_amd64.deb"
bin_name_rpm_string = "rocketchat-%s.x86_64.rpm"
package_name = "rocketchat"


def install():
    # Specific app values
    package_version = control.version.split("-")[0]
    bin_name_deb = bin_name_deb_string % package_version
    bin_name_rpm = bin_name_rpm_string % package_version

    # Installing the package
    if platform.linux_distribution()[0].lower() in ["debian", "ubuntu", "mint"]:
        print("Installing: %s" % bin_name_deb)
        install_deb(bin_name_deb)

    if platform.linux_distribution()[0].lower() in [
        "centos linux",
        "centos",
        "suse",
        "fedora",
        "redhat",
    ]:
        print("Installing: %s" % bin_name_rpm)
        install_yum(bin_name_rpm)


def uninstall():
    # Uninstalling the package
    if platform.linux_distribution()[0].lower() in ["debian", "ubuntu", "mint"]:
        print("Uninstalling: %s" % package_name)
        uninstall_apt(package_name)

    if platform.linux_distribution()[0].lower() in [
        "centos linux",
        "centos",
        "suse",
        "fedora",
        "redhat",
    ]:
        print("Uninstalling: %s" % package_name)
        uninstall_yum(package_name)


def update_package():
    print("Download/Update package content from upstream binary sources")

    # Getting proxy informations from WAPT settings
    proxy = {}
    if platform.system() == "Windows" and isfile(makepath(user_local_appdata(), "waptconsole", "waptconsole.ini")):
        proxywapt = inifile_readstring(
            makepath(user_local_appdata(), "waptconsole", "waptconsole.ini"),
            "global",
            "http_proxy",
        )
        if proxywapt:
            proxy = {"http": proxywapt, "https": proxywapt}

    # Specific app values
    app_name = control.name
    git_repo = "RocketChat/Rocket.Chat.Electron"
    url_api = "https://api.github.com/repos/%s/releases/latest" % git_repo
    bin_deb_end = bin_name_deb_string.split("%s")[-1]
    bin_rpm_end = bin_name_rpm_string.split("%s")[-1]

    # Getting latest informations from Github API
    json_load = json.loads(wgets(url_api, proxies=proxy))

    for download in json_load["assets"]:
        if download["browser_download_url"].endswith(bin_deb_end):
            url_dl_deb = download["browser_download_url"]
            break
    for download in json_load["assets"]:
        if download["browser_download_url"].endswith(bin_rpm_end):
            url_dl_rpm = download["browser_download_url"]
            break

    version = json_load["tag_name"]

    latest_bin_deb = bin_name_deb_string % version
    latest_bin_rpm = bin_name_rpm_string % version

    print("Latest " + app_name + " version is: " + version)
    print("Download url for APT-based systems is: %s" % url_dl_deb)
    print("Download url for YUM-based systems is: %s" % url_dl_rpm)

    # Downloading latest binaries
    if not isfile(latest_bin_deb):
        print("Downloading: " + latest_bin_deb)
        wget(url_dl_deb, latest_bin_deb, proxies=proxy)

    if not isfile(latest_bin_rpm):
        print("Downloading: " + latest_bin_rpm)
        wget(url_dl_rpm, latest_bin_rpm, proxies=proxy)

        # Changing version of the package
        pe = PackageEntry().load_control_from_wapt(".")
        pe.version = "%s-%s" % (version, int(pe.version.split("-", 1)[1]) + 1)
        pe.save_control_to_wapt(".")
        print("Changing version to " + pe.version + " in WAPT\\control")
        print("Update package done. You can now build-upload your package")
    else:
        print("This package is already up-to-date")

    # Deleting outdated binaries
    for bin_in_dir in glob.glob("*.deb") and glob.glob("*.rpm"):
        if bin_in_dir != latest_bin_rpm and bin_in_dir != latest_bin_deb:
            print("Outdated binary: " + bin_in_dir + " Deleted")
            remove_file(bin_in_dir)
