# -*- coding: utf-8 -*-
from setuphelpers import *
import os
import sys
import tempfile

"""     
    "update_package_dependencies": [
        "tis-wapt-selenium",
        "tis-firefox"
    ]
"""


def update_package():
    # Declaring local variables
    package_updated = False
    proxies = get_proxies_from_wapt_console()
    if not proxies:
        proxies = get_proxies()
    url = "https://www.citrix.com/downloads/workspace-app/windows/workspace-app-for-windows-latest.html"
    latest_bin = "CitrixWorkspaceApp.exe"

    # Deleting binaries
    remove_outdated_binaries("*")

    # Importing Selenium using package tis-wapt-selenium
    wapt_selenium_dir = os.sep.join([sys.executable.rsplit(os.path.sep, 1)[0].replace("/bin", "").replace("\\Scripts", ""), "selenium"])
    driver_path = makepath(wapt_selenium_dir, "geckodriver.exe")
    firefox_binary_path = makepath(programfiles, "Mozilla Firefox", "firefox.exe")
    profile_path = makepath(tempfile.gettempdir(), "gecko_mozprofile")
    driver_log_path = makepath(tempfile.gettempdir(), "geckodriver.log")
    user_agent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:109.0) Gecko/20100101 Firefox/115.0"
    sys.path.insert(0, wapt_selenium_dir)
    from selenium import webdriver
    from selenium.webdriver.firefox.firefox_binary import FirefoxBinary
    from selenium.webdriver.firefox.options import Options
    from selenium.webdriver.firefox.service import Service
    from selenium.webdriver.firefox.firefox_profile import FirefoxProfile
    from selenium.webdriver.common.by import By

    # Selenium WebDriver initialization
    driver_options = Options()
    # driver_options.headless = True
    # driver_options.set_capability("moz:firefoxOptions", {"args": ["-profile", profile_path]}) # unable to specify a profile
    driver_options.set_preference("general.useragent.override", user_agent)
    driver_options.set_preference("intl.accept_languages", "en-GB")
    driver_options.set_preference("browser.download.folderList", 2)
    driver_options.set_preference("browser.download.dir", basedir)
    driver_options.set_preference("browser.download.useDownloadDir", True)
    driver_options.set_preference("browser.helperApps.neverAsk.saveToDisk", "*/*")
    driver = webdriver.Firefox(options=driver_options, service=Service(driver_path, log_path=driver_log_path))
    driver.get(url)

    # Getting latest version with Selenium
    version = driver.find_element(By.XPATH, "/html/body/div[3]/div/div[1]/div[2]/div/div[3]/div/div[1]/div[1]/div/div[3]/div/p[2]").text
    version = version.split("Version: ", 1)[-1].split("(", 1)[0]

    # Downloading with Selenium
    driver.find_element(By.ID, "downloadcomponentid1").click()
    wait_download()
    driver.quit()
    wait_download()
    driver.quit()

    # Checking version from file
    if get_os_name() == "Windows" and "windows" in control.target_os.lower():
        version_from_file = get_version_from_binary(latest_bin)
        if Version(version_from_file, 4) == Version(version, 4):
            print(f"INFO: Binary file version ({version_from_file}) corresponds to online version ({version})")
        else:
            error(f"ERROR: Binary file version ({version_from_file}) do NOT corresponds to online version ({version})")

    # Changing version of the package
    if Version(version, 4) > Version(control.get_software_version(), 4):
        print("Software version updated (from: %s to: %s)" % (control.get_software_version(), Version(version)))
        package_updated = True
    else:
        print("Software version up-to-date (%s)" % Version(version))
    control.set_software_version(version)
    control.save_control_to_wapt()

    # Validating or not update-package-sources
    return package_updated

    # # Changing version of the package and validating update-package-sources
    # return complete_control_version(control, version)


def wait_download(delay=300, glob_str="*.exe"):
    loop = 0
    while not glob.glob(glob_str):
        loop += 5
        if loop > delay:
            break
        time.sleep(5)
        while glob.glob("*.part"):
            loop += 5
            if loop > delay:
                break
            time.sleep(5)
    time.sleep(5)
