#!waptpython
# -*- coding: utf-8 -*-
import os
import sys

sys.path.insert(0, os.path.dirname(os.path.realpath(__file__)))
from setupdevhelpers import *
from dev_utils import *

sys.path.insert(0, sys.executable.rsplit(os.path.sep, 1)[0].replace("/bin", "").replace("\\Scripts", ""))
from setuphelpers import *
import waptpackage
import argparse
import waptguihelper
import re
import json
import subprocess

######################################################### PARSER #########################################################

parser = argparse.ArgumentParser(description="Python script to unify multi WAPT Packages")
parser.add_argument(
    "--os",
    "-o",
    dest="os",
    default="ask",
    # action="store_true",
    help="Provide OS to interact with",
)
parser.add_argument(
    "--assume-yes",
    "--yes",
    "-y",
    dest="assume-yes",
    action="store_true",
    help="Assume yes to all queries and do not prompt",
)
parser.add_argument(
    "--silent",
    "-s",
    dest="silent",
    action="store_true",
    help="Assume no to unprovided queries and do not prompt",
)
parser.add_argument(
    "--base-files",
    dest="base-files",
    action="store_true",
    help="Overwrite older base files for specified OS",
)
parser.add_argument(
    "--unify-control",
    dest="unify-control",
    action="store_true",
    help="Unify control file for specified OS",
)
parser.add_argument(
    "--overwrite-older-py-files",
    dest="overwrite-older-py-files",
    action="store_true",
    help="Overwrite older setup.py and update_package.py files with the most recent in the OS subfolders",
)
parser.add_argument(
    "--complete-control",
    dest="complete-control",
    action="store_true",
    help="Helper to complete the control file if needed",
)
args = parser.parse_args()
options = vars(args)

answers = 0
answer_base_files = "n"
answer_unify_control = "n"
answer_erase_py_files = "n"
answer_complete_control = "n"

if options["os"] == "ask":
    if options["silent"]:
        answer_os_interaction = "n"
    else:
        answer_os_interaction = input("Interact on all OSes or specify? (y/...): ")
# elif options["os"] == "all":
#     answer_os_interaction = "y"
else:
    answer_os_interaction = options["os"]

if options["assume-yes"]:
    answers += 1
    answer_base_files = "y"
    answer_unify_control = "y"
    answer_erase_py_files = "y"
else:
    if options["base-files"]:
        answers += 1
        answer_base_files = "y"
    if options["unify-control"]:
        answers += 1
        answer_unify_control = "y"
    if options["overwrite-older-py-files"]:
        answers += 1
        answer_erase_py_files = "y"
    if options["complete-control"]:
        answers += 1
        answer_complete_control = "y"


if options["silent"]:
    answers += 1

if answers == 0:
    # TODO use waptguihelper
    # answer_os_interaction = input("Interact on all OSes or specify? (y/...): ")
    answer_base_files = input("Generate base files for current OS? (y/n): ")
    answer_unify_control = input("Unify control files? (y/n): ")
    answer_erase_py_files = input("Erase same OS .py files with the most recent? (y/n): ")
    answer_translate_control = input("Translate control file with DeepL? (y/n): ")
    answer_complete_control = waptguihelper.message_dialog("package_template.py", "Complete control file with helper?", waptguihelper.MB_YESNO)
    if answer_complete_control == 6:
        answer_complete_control = "y"
    else:
        answer_complete_control = "n"
######################################################### PARSER #########################################################


def control_file_unified(
    dst,
    src,
    answer_os_interaction,
    keys_to_clear_list,
    operating_system=None,
    arch=None,
    lang=None,
):
    control_src_dict = get_control_dict(src)
    if isfile(makepath(dst, "WAPT", "control")):
        control_dst_dict = get_control_dict(dst)
    else:
        control_dst_dict = control_src_dict
    control_uni_dict = {}

    for key in control_src_dict.keys():
        if control_src_dict[key]:
            control_uni_dict[key] = control_src_dict[key]
        else:
            control_uni_dict[key] = control_dst_dict[key]

    if control_uni_dict["version"] == "0":
        control_uni_dict["version"] = control_dst_dict["version"]
    if not "-" in control_uni_dict["version"]:
        control_uni_dict["version"] = control_uni_dict["version"] + "-0"
    if not arch:
        arch = control_dst_dict["architecture"]
    if not lang:
        lang = "all"

    control_uni_dict["architecture"] = arch
    control_uni_dict["locale"] = lang

    if operating_system is not None:
        if len(control_dst_dict["target_os"].split(",")) > 1 or control_dst_dict["target_os"].startswith(operating_system):
            control_uni_dict["target_os"] = control_dst_dict["target_os"]
        else:
            if operating_system.startswith(control_dst_dict["target_os"]):
                if not control_dst_dict["min_os_version"] or control_dst_dict["max_os_version"]:
                    control_uni_dict["target_os"] = operating_system
            else:
                control_uni_dict["target_os"] = operating_system
    control_uni_dict["target_os"] = control_uni_dict["target_os"].lower()

    # limit changes if different os_dir
    if src.split(os.sep)[0] != dst.split(os.sep)[0]:
        control_uni_dict["min_os_version"] = control_dst_dict["min_os_version"]
        control_uni_dict["max_os_version"] = control_dst_dict["max_os_version"]
        control_uni_dict["depends"] = control_dst_dict["depends"]
        control_uni_dict["conflicts"] = control_dst_dict["conflicts"]
    # force changes if same os_dir
    else:
        control_uni_dict["installed_size"] = control_src_dict["installed_size"]
        control_uni_dict["depends"] = control_src_dict["depends"]
        control_uni_dict["conflicts"] = control_src_dict["conflicts"]
    # force some changes if os is all
    if answer_os_interaction == "all":
        control_uni_dict["depends"] = control_src_dict["depends"]
        control_uni_dict["conflicts"] = control_src_dict["conflicts"]
        control_uni_dict["installed_size"] = control_src_dict["installed_size"]

    if control_uni_dict["maturity"] == "":
        control_uni_dict["maturity"] = "PROD"

    for key in keys_to_clear_list:
        del control_uni_dict[key]

    # # Increment if any translation
    # if descriptions_updated:
    #     new_pkg_version = int(control_dst_dict["version"].split("-", 1)[-1]) + 1
    #     control_uni_dict["version"] = "%s-%s" % (control_dst_dict["version"].split("-", 1)[0], new_pkg_version)

    new_pkg_entry = waptpackage.PackageEntry().load_control_from_dict(control_uni_dict)
    new_pkg_entry.save_control_to_wapt(dst)


def get_newer_file(pkg_dir_list, makepath_function, os_based=False, pkg_os_dir=None):
    if os_based and pkg_os_dir is None:
        error("Please provide pkg_os_dir")
    newest_file_date = 0
    newest_file = None
    if isfile(makepath_function("template")):
        pkg_dir_list = pkg_dir_list + ["template"]

    for pkg_dir in pkg_dir_list:
        if pkg_dir == "template":
            pass
        elif os_based:
            if (
                not [i for i in re.split(r"(\d+)", pkg_dir.split(os.sep)[0]) if i][0] == [i for i in re.split(r"(\d+)", pkg_os_dir) if i][0]
                and not pkg_dir.split(os.sep)[0].split("-")[0] == pkg_os_dir.split("-")[0]
            ):
                continue
        if isfile(makepath_function(pkg_dir)):
            file_date = os.path.getmtime(makepath_function(pkg_dir))
            if file_date > newest_file_date:
                newest_file_date = file_date
                newest_file = makepath_function(pkg_dir)
    return newest_file


def overwrite_older_file(pkg_dir_list, pkg_os_dir, makepath_function, os_based=False):
    file_to_overwrite = makepath_function(pkg_dir)
    overwritten_file = get_newer_file(pkg_dir_list, makepath_function, os_based, pkg_os_dir)
    if overwritten_file is not None and overwritten_file != file_to_overwrite:
        filecopyto(overwritten_file, file_to_overwrite)
    return overwritten_file


######################################################### Variables #########################################################

arch_list = ["all", "x64", "x86", "arm64"]
lang_list = [
    "all",
    "en",
    "fr",
    "pl",
    "de",
    "es",
    "pt",
    "it",
    "nl",
    "ru",
]
keys_to_clear_list = [
    "signature_date",
    "signed_attributes",
    "package_uuid",
    "signature",
    "signer",
    "signer_fingerprint",
    "icon_sha256sum",
]
pkg_dir_list = []
descriptions_updated = False
deepl_auth_key = ""

######################################################### Code #########################################################

template_wapt_dir = makepath(r"template\WAPT")
if not isdir(template_wapt_dir):
    mkdirs(template_wapt_dir)

# Getting OSes
for get_pkg_s_py in glob.glob("**/setup.py", recursive=True):
    pkg_os_dir = get_pkg_s_py.split(os.sep + "setup.py", 1)[0]
    if pkg_os_dir == "template":
        continue
    elif pkg_os_dir == "setup.py":
        pkg_os_dir = "."

    pkg_dir_list.append(pkg_os_dir)


# Package interactions
for pkg_dir in pkg_dir_list:
    pkg_os = get_control_dict(pkg_dir)["target_os"]
    pkg_os_dir = pkg_dir.split(os.sep)[0]
    if not pkg_os:
        pkg_os = pkg_os_dir
    control_path = control_makepath(pkg_dir)
    control_to_clean = False

    if pkg_dir == "template":
        continue

    if answer_base_files == "y" and answer_os_interaction in [pkg_os, "y", "all"]:
        # changelog_txt
        file_path = changelog_txt_makepath(pkg_dir)
        file_sha256 = get_file_sha256(file_path)
        if answer_os_interaction == "all":
            newer_file = overwrite_older_file(pkg_dir_list, pkg_os_dir, changelog_txt_makepath, os_based=False)
        else:
            newer_file = overwrite_older_file(pkg_dir_list, pkg_os_dir, changelog_txt_makepath, os_based=True)

        if file_sha256 != get_file_sha256(file_path):
            print("Overwritten: %s with: %s" % (file_path, newer_file))

        # icon_png
        file_path = icon_png_makepath(pkg_dir)
        file_sha256 = get_file_sha256(file_path)
        if answer_os_interaction == "all":
            newer_file = overwrite_older_file(pkg_dir_list, pkg_os_dir, icon_png_makepath, os_based=False)
        else:
            newer_file = overwrite_older_file(pkg_dir_list, pkg_os_dir, icon_png_makepath, os_based=True)
        if file_sha256 != get_file_sha256(file_path):
            print("Overwritten: %s with: %s" % (file_path, newer_file))

    # control file
    lang = None
    arch = None
    for dir_name in pkg_dir.split(os.sep):
        if lang is None:
            if dir_name in lang_list:
                lang = dir_name
        if arch is None:
            if dir_name in arch_list:
                arch = dir_name

    if answer_unify_control == "y":
        if answer_os_interaction in [pkg_os, "y", "all"]:
            # control
            if answer_os_interaction == "all":
                newer_control_file = get_newer_file(pkg_dir_list, control_makepath, os_based=False, pkg_os_dir=pkg_os_dir)
            else:
                newer_control_file = get_newer_file(pkg_dir_list, control_makepath, os_based=True, pkg_os_dir=pkg_os_dir)
            control_file_sha256 = get_file_sha256(control_path)

            # Unifying control file in the package
            if newer_control_file:
                control_file_unified(
                    pkg_dir,
                    newer_control_file,
                    answer_os_interaction=answer_os_interaction,
                    keys_to_clear_list=keys_to_clear_list,
                    operating_system=pkg_os,
                    arch=arch,
                    lang=lang,
                )
                if control_file_sha256 != get_file_sha256(control_path):
                    print("Unified control file: %s with: %s" % (control_path, newer_control_file))
            else:
                control_to_clean = True
    else:
        if isfile(control_path) and not descriptions_updated:
            control_to_clean = True

    if control_to_clean:
        control_file_sha256 = get_file_sha256(control_path)
        control_file_unified(
            pkg_dir,
            pkg_dir,
            answer_os_interaction=answer_os_interaction,
            keys_to_clear_list=keys_to_clear_list,
            operating_system=get_control_dict(pkg_dir)["target_os"],
            arch=get_control_dict(pkg_dir)["architecture"],
            lang=get_control_dict(pkg_dir)["locale"],
        )
        if control_file_sha256 != get_file_sha256(control_path):
            print("Cleaned control file: %s" % control_path)

    # Erase same OS .py files with the most recent
    if answer_erase_py_files == "y" and answer_os_interaction in [pkg_os, "y", "all"]:
        # setup_py
        file_path = setup_py_makepath(pkg_dir)
        file_sha256 = get_file_sha256(file_path)
        if answer_os_interaction == "all":
            newer_file = overwrite_older_file(pkg_dir_list, pkg_os_dir, setup_py_makepath, os_based=False)
        else:
            newer_file = overwrite_older_file(pkg_dir_list, pkg_os_dir, setup_py_makepath, os_based=True)
        if file_sha256 != get_file_sha256(file_path):
            print("Overwritten: %s with: %s" % (file_path, newer_file))

        # update_package_py
        file_path = update_package_py_makepath(pkg_dir)
        file_sha256 = get_file_sha256(file_path)
        if answer_os_interaction == "all":
            newer_file = overwrite_older_file(pkg_dir_list, pkg_os_dir, update_package_py_makepath, os_based=False)
        else:
            newer_file = overwrite_older_file(pkg_dir_list, pkg_os_dir, update_package_py_makepath, os_based=True)
        if file_sha256 != get_file_sha256(file_path):
            print("Overwritten: %s with: %s" % (file_path, newer_file))


# Asking missing control values
if answer_complete_control == "y":
    template_control = control_makepath("template")
    if isfile(template_control):
        remove_file(template_control)

    newer_control_file = get_newer_file(pkg_dir_list, control_makepath)
    provided_control_dict = {}

    if get_control_dict(newer_control_file)["name"] == "":
        name = waptguihelper.input_dialog("package_template.py", "Package Name is missing, please provide it:", "")
        provided_control_dict.update({"name": name})
        new_template_control = waptpackage.PackageEntry().load_control_from_dict(provided_control_dict)
        new_template_control.save_control_to_wapt("template")

    if get_control_dict(newer_control_file)["categories"] == "":
        categories = waptguihelper.grid_dialog(
            "Select %s categories" % get_control_dict(newer_control_file)["package"],
            [
                "Internet",
                "Utilities",
                "Messaging",
                "Security",
                "System and network",
                "Media",
                "Development",
                "Office",
                "Drivers",
                "Education",
                "Configuration",
                "CAD",
                "Template",
                "Dependency",
                "Extension",
            ],
            waptguihelper.GRT_SELECTED,
        )
        provided_control_dict.update({"categories": ",".join([a["unknown"] for a in categories])})
        new_template_control = waptpackage.PackageEntry().load_control_from_dict(provided_control_dict)
        new_template_control.save_control_to_wapt("template")

    if get_maintainer_name() is None:
        maintainer = waptguihelper.input_dialog("package_template.py", "Who is in charge of this package maintenance:", get_current_user())
        save_maintainer_name(maintainer)
    else:
        maintainer = get_maintainer_name()

    provided_maintainers_list = []
    if not get_control_dict(newer_control_file)["maintainer"] == "":
        for m in get_control_dict(newer_control_file)["maintainer"].split(","):
            if not m.lower() == get_current_user().lower():
                provided_maintainers_list.append(m)

    if "WAPT Team".lower() not in [s.lower() for s in provided_maintainers_list]:
        provided_maintainers_list.append("WAPT Team")
    if "Tranquil IT".lower() not in [s.lower() for s in provided_maintainers_list]:
        provided_maintainers_list.append("Tranquil IT")
    if maintainer.lower() not in [s.lower() for s in provided_maintainers_list]:
        provided_maintainers_list.append(maintainer)
    provided_control_dict.update({"maintainer": ",".join(provided_maintainers_list)})
    new_template_control = waptpackage.PackageEntry().load_control_from_dict(provided_control_dict)
    new_template_control.save_control_to_wapt("template")

    if get_control_dict(newer_control_file)["min_wapt_version"] == "":
        min_wapt_version = waptguihelper.input_dialog(
            "package_template.py",
            "Prodive the minimum WAPT compatible version for this package, it is recommended to provide at least the latest maintained WAPT version, refer to: https://www.wapt.fr/doc/wapt-enterprise.html",
            "",
        )
        provided_control_dict.update({"min_wapt_version": min_wapt_version})
        new_template_control = waptpackage.PackageEntry().load_control_from_dict(provided_control_dict)
        new_template_control.save_control_to_wapt("template")

    if get_control_dict(newer_control_file)["installed_size"] == "" or get_control_dict(newer_control_file)["impacted_process"] == "":
        program_dir = waptguihelper.directory_dialog("Please select the installed location of the software (do not specify if not relevant)")

    if get_control_dict(newer_control_file)["installed_size"] == "" and program_dir is not None:
        installed_size = get_size(program_dir)
        provided_control_dict.update({"installed_size": installed_size})
        new_template_control = waptpackage.PackageEntry().load_control_from_dict(provided_control_dict)
        new_template_control.save_control_to_wapt("template")

    if get_control_dict(newer_control_file)["impacted_process"] == "" and program_dir is not None:
        exe_list = []
        for exe_path in glob.glob("%s/**/*.exe" % (program_dir), recursive=True):
            exe = exe_path.split("\\")[-1].split(".exe")[0]
            exe_list.append(exe)
        impacted_process = waptguihelper.grid_dialog(
            "Select (by clicking on them) %s impacted processes" % get_control_dict(newer_control_file)["package"],
            exe_list,
            waptguihelper.GRT_SELECTED,
        )

        provided_control_dict.update({"impacted_process": ",".join([a["unknown"] for a in impacted_process])})
        new_template_control = waptpackage.PackageEntry().load_control_from_dict(provided_control_dict)
        new_template_control.save_control_to_wapt("template")

    # Apply control changes
    print(str(run(f'"{sys.executable}" "{os.path.basename(__file__)}" --os y --unify-control')))
    if isfile(template_control):
        remove_file(template_control)

    # Add icon.png
    newer_icon_png = get_newer_file(pkg_dir_list, icon_png_makepath)
    # TODO ask differently since it is not convenient for non-application package
    if not newer_icon_png:
        icon_exe_to_extract = waptguihelper.filename_dialog("Please provide the binary to extract the icon", "", "", "EXE Files|*.exe")
        extracted_icon = extract_icon(icon_exe_to_extract, icon_png_makepath("template"))
        if not extracted_icon:
            new_icon_png = waptguihelper.filename_dialog(
                "The extraction failed please provide an icon.png for the package", "", "icon.png", "PNG Files|*.png"
            )
            filecopyto(new_icon_png, icon_png_makepath("template"))
        else:
            new_icon_png = icon_png_makepath("template")
        reduce_if_needed(icon_png_makepath("template"))

    print(str(run(f'"{sys.executable}" "{os.path.basename(__file__)}" --os y --base-files')))


# Formatting code
formatting_print = str(run_notfatal("waptpython -m black -l 150 ."))
if "reformatted" in formatting_print:
    print(formatting_print.split("\r\nAll done!")[0].split("\nAll done!")[0])
